//
//  LoginViewController.swift
//  TalkPlusSampleSwift
//
//  Created by hnroh on 2021/01/06.
//

import UIKit
import HiveTalkPlus
import HIVEService

class LoginViewController: UIViewController {
    
    let HIVE_AUTO_LOGIN_MODE = false
    @IBOutlet weak var titleLabel: UILabel!
    
    override func viewDidLoad() {
        super.viewDidLoad()
            
        // UI
        titleLabel.text = """
                          HiveTalkPlus
                          \(HiveTalkPlus.getVersion())
                          """
        
        // Init
        if !HIVE_AUTO_LOGIN_MODE {
            initialize()
            return
        }
        
        hiveAutoLogin { playerInfo in
            // SUCCESS
            self.initialize()
        } failureHandler: {
            // FAILURE
        }
    }
    
    // MARK: - Action
    @IBAction func loginWithAuthV4Action(_ sender: Any) {
        loginWithAuthV4()
    }
    
    @IBAction func goBacktoMainAction(_ sender: Any) {
        self.dismiss(animated: true, completion: nil)
    }
    
    private func initialize() {
        HiveTalkPlus.initWithHandler { isInit, message in
            LogManager.shared.addLog("HiveTalkPlus.initWithAppId called. message = \(message)")
        }
    }
    
    private func hiveAutoLogin(successHandler: ((PlayerInfo?)->Void)?, failureHandler: (()->Void)?) {
        AuthV4Interface.setup { result, isAutoSignIn, did, providerTypeList in
            
            LogManager.shared.addLog("AuthV4Interface.setup result = \(result.description)")
            
            guard result.isSuccess() else {
                LogManager.shared.addLog("AuthV4Interface.setup failure")
                failureHandler?()
                return
            }
            
            if isAutoSignIn {
                AuthV4Interface.signIn(.Auto) { signIn_result, playerInfo in
                    
                    LogManager.shared.addLog("AuthV4Interface.signIn result = \(signIn_result.description)")
                    
                    guard signIn_result.isSuccess() else {
                        LogManager.shared.addLog("AuthV4Interface.signIn failure")
                        failureHandler?()
                        return
                    }
                    
                    successHandler?(playerInfo)
                }
                return
            }
            
            AuthV4Interface.showSignIn { signIn_result, playerInfo in
                
                LogManager.shared.addLog("AuthV4Interface.showSignIn result = \(signIn_result.description)")
                
                guard signIn_result.isSuccess() else {
                    LogManager.shared.addLog("AuthV4Interface.showSignIn failure")
                    failureHandler?()
                    return
                }
                
                successHandler?(playerInfo)
            }
        }
    }
    
    // MARK: - Login
    private func loginWithAuthV4() {
        AuthV4Interface.getHiveTalkPlusLoginToken { result, token in
            LogManager.shared.addLog("AuthV4Interface.getHiveTalkPlusLoginToken callback")
            LogManager.shared.addLog("result = \(result.description)")

            guard result.isSuccess(), let token = token, !token.isEmpty else {
                LogManager.shared.addLog("token = nil or empty")
                return
            }

            LogManager.shared.addLog("AuthV4Interface.getPlayerInfo called.");
            guard let playerInfo = AuthV4Interface.getPlayerInfo() else {
                LogManager.shared.addLog("playerInfo = nil")
                return
            }

            let playerId = "\(playerInfo.playerID)"
            let playerName = playerInfo.playerName
            let playerImageUrl = playerInfo.playerImageURL

            LogManager.shared.addLog("""
                                    token = \(token)
                                    playerInfo = \(playerInfo.description)
                                    """)

            HiveTalkPlus.login(withToken: token, userId: playerId, username: playerName, profileImageUrl: playerImageUrl, metaData: nil, success: { [weak self] user in

                guard let user = user, let userId = user.getId(),
                      let userName = user.getUsername()
                else {
                    LogManager.shared.addLog("Invalid user info.")
                    return
                }

                UserDefaults.standard.set(userId, forKey: "KeyUserID")
                UserDefaults.standard.set(userName, forKey: "KeyUserName")
                UserDefaults.standard.synchronize()

                self?.performSegue(withIdentifier: "SegueMain", sender: nil)

                LogManager.shared.addLog("user info = \(user.description)")

            }, failure: { [weak self] errorCode, error in
                // log
                self?.showToast("로그인에 실패하였습니다")
                LogManager.shared.addFailureLog(api: "login", errorCode: errorCode, error: error)
            })

            LogManager.shared.addLog("HiveTalkPlus.login called.")
        }
        LogManager.shared.addLog("AuthV4Interface.getHiveTalkPlusLoginToken called.")
    }
}
